@@ -7,6 +7,8 @@ use Apache::Constants qw(OK DECLINED);
 use Apache::Connection ();
 use Apache::Server ();
 
+eval { require Apache::Table; };
+
 @Apache::EXPORT_OK = qw(exit warn);
 
 *import = \&Exporter::import;
@@ -46,7 +48,7 @@ sub parse_args {
 sub content {
     my($r) = @_;
     my $ct = $r->header_in("Content-type") || "";
-    return unless $ct eq "application/x-www-form-urlencoded";
+    return unless $ct =~ m!^application/x-www-form-urlencoded!;
     my $buff;
     $r->read($buff, $r->header_in("Content-length"));
     parse_args(wantarray, $buff);
@@ -162,6 +164,7 @@ sub send_cgi_header {
     my($r, $headers) = @_;
     my $dlm = "\015?\012"; #a bit borrowed from LWP::UserAgent
     my($key, $val);
+    local $_;
     while(($_, $headers) = split /$dlm/, $headers, 2) {
 	#warn "hunk=`$_'\n";
 	#warn "rest=`$headers'\n";
@@ -186,11 +189,6 @@ sub send_cgi_header {
     }
 }
 
-sub TIEHANDLE {
-    my($class, $r) = @_;
-    $r ||= Apache->request;
-}
-
 1;
 
 __END__
@@ -214,9 +212,9 @@ the description of the Apache C API at http://www.apache.org/docs/.
 
 The request object holds all the information that the server needs to
 service a request.  Apache B<Perl*Handler>s will be given a reference to the
-request object as parameter and may choose update or use it in various
+request object as parameter and may choose to update or use it in various
 ways.  Most of the methods described below obtain information from or
-updates the request object.
+update the request object.
 The perl version of the request object will be blessed into the B<Apache> 
 package, it is really a C<request_rec*> in disguise.
 
@@ -337,7 +335,7 @@ The number of bytes sent to the client, handy for logging, etc.
 
 =item $r->the_request
 
-The request line send by the client, handy for logging, etc.
+The request line sent by the client, handy for logging, etc.
 
 =item $r->proxyreq
 
@@ -356,10 +354,19 @@ The $r->protocol method will return a string identifying the protocol
 that the client speaks.  Typical values will be "HTTP/1.0" or
 "HTTP/1.1".
 
+=item $r->hostname
+
+Returns the server host name, as set by full URI or Host: header.
+
+=item $r->request_time
+
+Returns the time that the request was made.  The time is the local unix
+time in seconds since the epoch.
+
 =item $r->uri( [$uri] )
 
-The $r->uri method will return the requested URI, optionally changing
-it with the first argument.
+The $r->uri method will return the requested URI minus optional query
+string, optionally changing it with the first argument.
 
 =item $r->filename( [$filename] )
 
@@ -373,7 +380,7 @@ The $r->path_info method will return what is left in the path after the
 I<URI --E<gt> filename> translation, optionally changing it with the first 
 argument if you happen to be doing the translation.
 
-=item $r->args
+=item $r->args( [$query_string] )
 
 The $r->args method will return the contents of the URI I<query
 string>.  When called in a scalar context, the entire string is
@@ -383,6 +390,9 @@ I<value> pairs are returned, i.e. it can be used like this:
    $query = $r->args;
    %in    = $r->args;
 
+$r->args can also be used to set the I<query string>. This can be useful
+when redirecting a POST request.
+
 =item $r->headers_in
 
 The $r->headers_in method will return a %hash of client request
@@ -417,7 +427,7 @@ This method is used to read data from the client,
 looping until it gets all of C<$bytes_to_read> or a timeout happens.
 
 In addition, this method sets a timeout before reading with
-C<$r-E<gt>hard_timeout>
+C<$r-E<gt>hard_timeout>.
 
 =item $r->get_remote_host
 
@@ -467,12 +477,12 @@ this variable.
 =item $c->remote_ip
 
 The dotted decimal representation of the remote client's IP address.
-This is set by then server when the connection record is created so
+This is set by the server when the connection record is created so
 is always defined.
 
 You can also set this value by providing an argument to it. This is
-helpful if your server is behind a squid accelerator proxy which add
-a HTTP_X_FORWARDED_FOR header.
+helpful if your server is behind a squid accelerator proxy which adds
+a X-Forwarded-For header.
 
 =item $c->local_addr
 
@@ -517,10 +527,11 @@ If the configuration directive B<IdentityCheck> is set to off: then
 C<$r-E<gt>get_remote_logname> does nothing and C<$c-E<gt>remote_logname> is
 always undefined.
 
-=item $c->user 
+=item $c->user( [$user] )
 
 If an authentication check was successful, the authentication handler
-caches the user name here.
+caches the user name here. Sets the user name to the optional first
+argument.
 
 =item $c->auth_type
 
@@ -531,6 +542,15 @@ C<$c-E<gt>user>, if any.
 
 Returns true if the client stopped talking to us.
 
+=item $c->fileno( [$direction] )
+
+Returns the client file descriptor. If $direction is 0, the input fd
+is returned. If $direction is not null or ommitted, the output fd is
+returned.
+
+This can be used to detect client disconnect without doing any I/O,
+e.g. using IO::Select.
+
 =back
 
 =back
@@ -548,11 +568,13 @@ Returns the value of a per-directory variable specified by the
 C<PerlSetVar> directive.
 
    # <Location /foo/bar>
-   # SetPerlVar  Key  Value
+   # PerlSetVar  Key  Value
    # </Location>
 
    my $val = $r->dir_config('Key');
 
+Keys are case-insensitive.
+
 Will return a I<HASH> reference blessed into the
 I<Apache::Table> class when called in a scalar context with no
 "key" argument. See I<Apache::Table>.
@@ -641,6 +663,10 @@ Returns true if this is a virtual server.
 
 Returns the wild-carded names for ServerAlias servers. 
 
+=item $s->dir_config( $key )
+
+Alias for Apache::dir_config.
+
 =item $s->warn
 
 Alias for Apache::warn.
@@ -649,6 +675,64 @@ Alias for Apache::warn.
 
 Alias for Apache::log_error.
 
+=item $s->uid
+
+Returns the numeric user id under which the server answers requests.
+This is the value of the User directive.
+
+=item $s->gid
+
+Returns the numeric group id under which the server answers requests.
+This is the value of the Group directive.
+
+=item $s->loglevel
+
+Returns the value of the current LogLevel. This method is added by
+the Apache::Log module, which needs to be pulled in.
+
+    use Apache::Log;
+    print "LogLevel = ", $s->loglevel;
+
+If using Perl 5.005+, the following constants are defined (but not
+exported):
+
+    Apache::Log::EMERG
+    Apache::Log::ALERT
+    Apache::Log::CRIT
+    Apache::Log::ERR
+    Apache::Log::WARNING
+    Apache::Log::NOTICE
+    Apache::Log::INFO
+    Apache::Log::DEBUG
+
+=item $r->get_handlers( $hook )
+
+Returns a reference to a list of handlers enabled for $hook. $hook is
+a string representing the phase to handle. The returned list is a list
+of references to the handler subroutines.
+
+	$list = $r->get_handlers( 'PerlHandler' );
+
+=item $r->set_handlers( $hook, [\&handler, ... ] )
+
+Sets the list if handlers to be called for $hook. $hook is a string
+representing the phase to handle. The list of handlers is an anonymous
+array of code references to the handlers to install for this request
+phase. The special list [ \&OK ] can be used to disable a particular
+phase.
+
+	$r->set_handlers( PerlLogHandler => [ \&myhandler1, \&myhandler2 ] );
+	$r->set_handlers( PerlAuthenHandler => [ \&OK ] );
+
+=item $r->push_handlers( $hook, \&handler )
+
+Pushes a new handler to be called for $hook. $hook is a string
+representing the phase to handle. The handler is a reference to a
+subroutine to install for this request phase. This handler will be
+called before any configured handlers.
+
+	$r->push_handlers( PerlHandler => \&footer);
+
 =back
 
 =head1 SETTING UP THE RESPONSE
@@ -690,7 +774,7 @@ for the realm defined by the configuration directive C<AuthName>.
 
 Set the handler for a request.
 Normally set by the configuration directive C<AddHandler>.
-  
+
    $r->handler( "perl-script" );
 
 =item $r->notes( $key, [$value] )
@@ -962,8 +1046,8 @@ $r->send_http_header().  Example of use:
 
    $r->send_cgi_header(<<EOT);
    Location: /foo/bar
-   Content-type: text/html 
-   
+   Content-type: text/html
+
    EOT
 
 =back
